#-----------------------------------------------------------------------------
#
#   PyWeek 11 - Game
#
#-----------------------------------------------------------------------------

from itertools import cycle
from random import shuffle
from pygame import Rect
from albow.resource import get_image
from albow.image_array import get_image_array
from humerus.hgame import HGame

NUM_STACKS = 6
STACK_WIDTH = 110
STACKS_LEFT = 10
STACKS_TOP = 10
#CARD_WIDTH = 100
#CARD_HEIGHT = 150
CARD_OVERLAP = 20
CARD_MULTIPLICITY = 4

card_kinds = [
	('cat', 'mouse', 'bird', 'cage'),
	('net', 'butterfly', 'dog', 'fly'),
	('frog', 'spider', 'worm'),
]

catch_map = {
	'cat': ['mouse', 'bird'],
	'dog': ['cat'],
	'spider': ['fly'],
	'frog': ['fly'],
	'net': ['butterfly'],
	'cage': ['bird'],
	'bird': ['worm'],
}

card_images = get_image_array("card_faces.png",
	(len(card_kinds), len(card_kinds[0])))

#-----------------------------------------------------------------------------

def create_cards():
	cards = []
	for i, row in enumerate(card_kinds):
		for j, kind in enumerate(row):
			image = card_images[i, j]
			for n in xrange(CARD_MULTIPLICITY):
				cards.append(Card(kind, image))
	return cards

#-----------------------------------------------------------------------------

class Card(object):

	def __init__(self, kind, image):
		self.kind = kind
		self.image = image
		self.rect = Rect(image.get_rect())
		self.stack = None
		self.offset = (0, 0)
	
	def __repr__(self):
		return "<card %s at 0x%x>" % (self.kind, id(self))
	
	def put_on_stack(self, new_stack):
		self.remove_from_stack()
		self.stack = new_stack
		if new_stack:
			n = len(new_stack.cards)
			new_stack.cards.append(self)
			self.rect.topleft = (new_stack.left, new_stack.top + n * CARD_OVERLAP)
	
	def remove_from_stack(self):
		old_stack = self.stack
		if old_stack:
			old_stack.cards.remove(self)
	
	def effective_rect(self):
		dx, dy = self.offset
		return self.rect.move(dx, dy)

#-----------------------------------------------------------------------------

class Stack(object):

	def __init__(self, number):
		self.number = number
		self.left = STACKS_LEFT + number * STACK_WIDTH
		self.top = STACKS_TOP
		self.cards = []
	
	def top_card(self):
		cards = self.cards
		if cards:
			return cards[-1]
	
#-----------------------------------------------------------------------------

class GameState(object):
	#  stacks          [Stack]
	#  dragging_card   Card or None
	
	dragging_card = None

	def __init__(self):
		self.stacks = [Stack(i) for i in xrange(NUM_STACKS)]
		self.cards = create_cards()
		self.deal_cards()
	
	def deal_cards(self):
		shuffle(self.cards)
		for card, stack in zip(self.cards, cycle(self.stacks)):
			card.put_on_stack(stack)
	
	def drop_card(self, dropped_card):
		p = dropped_card.effective_rect().center
		for stack in self.stacks:
			top_card = stack.top_card()
			if top_card and top_card.rect.collidepoint(p):
				self.attempt_catch(dropped_card, top_card)
			else:
				self.return_to_start_position(dropped_card)
	
	def attempt_catch(self, card1, card2):
		#  Try to use card1 to catch card2
		catches = catch_map.get(card1.kind)
		if catches and card2.kind in catches:
			self.remove_from_play(card1)
			self.remove_from_play(card2)
		else:
			self.return_to_start_position(card1)
	
	def return_to_start_position(self, card):
		card.offset = (0, 0)
	
	def remove_from_play(self, card):
		card.remove_from_stack()
		self.cards.remove(card)

	def game_is_completed(self):
		return not self.cards

#-----------------------------------------------------------------------------

class Game(HGame):

	def new_state(self):
		return GameState()
	
	def game_is_completed(self):
		return self.state.game_is_completed()

#-----------------------------------------------------------------------------

game = Game()
