#-----------------------------------------------------------------------------
#
#   PyWeek 11 - Playing screen
#
#-----------------------------------------------------------------------------

from humerus.main_commands import MainCommandScreen
from game import game

class PlayScreen(MainCommandScreen):

	def draw(self, surf):
		state = game.state
		top_card = state.dragging_card
		for stack in state.stacks:
			for card in stack.cards:
				if card is not top_card:
					self.draw_card(surf, card)
		if top_card:
			self.draw_card(surf, top_card)
	
	def draw_card(self, surf, card):
		x, y = card.rect.topleft
		dx, dy = card.offset
		surf.blit(card.image, (x + dx, y + dy))

	def mouse_down(self, e):
		state = game.state
		p = e.pos
		for stack in state.stacks:
			card = stack.top_card()
			if card and card.rect.collidepoint(p):
				state.dragging_card = card
				self.drag_start = p
				card.offset = (0, 0)
				return

	def mouse_drag(self, e):
		self.drag(e, False)
	
	def mouse_up(self, e):
		self.drag(e, True)

	def drag(self, e, release):
		state = game.state
		card = state.dragging_card
		if card:
			x0, y0 = self.drag_start
			mx, my = e.pos
			card.offset = (mx - x0, my - y0)
			if release:
				state.dragging_card = None
				state.drop_card(card)
